from setuptools import setup
from setuptools.command.build_py import build_py
import subprocess
import os
import requests
import semver

        
vers = requests.get(f"https://{os.environ.get('NETDB_ENDPOINT')}").json()[0]
largest_ver = None
for v in vers:
    sem_v = semver.Version.parse(v['numeric'])
    if largest_ver is None or largest_ver < sem_v:
        largest_ver = sem_v 

class APIGenBuild(build_py):
    def run(self):
        build_versions = [f'{v["major"]}.{v["minor"]}' for v in vers]
        default_version = f'{largest_ver.major}.0'
        environ = os.environ.copy()
        for version in build_versions:
            target_dir = os.path.join(self.build_lib, 'netdb_client', f"api{version.replace('.', '')}")
            self.mkpath(target_dir)
            environ['NETDB_VERSION'] = version
            subprocess.run(['net-api-generator', 'python', f'--output-dir={target_dir}'], check=True, env=environ)
        target_dir = os.path.join(self.build_lib, 'netdb_client')
        environ['NETDB_VERSION'] = default_version
        subprocess.run(['net-api-generator', 'python', f'--output-dir={target_dir}'], check=True, env=environ)
        build_py.run(self)

setup(
    name='netdb_client',
    version=f'{str(largest_ver)}+{os.environ.get("CI_COMMIT_BRANCH", "local")}-{os.environ.get("CI_COMMIT_SHORT_SHA", "HEAD")}',
    author='NETVS-Team <netvs@scc.kit.edu>',
    description='This is a meta package to install the automatically generated NET-API definitionsi for the currently supported API versions.',
    url='https://git.scc.kit.edu/scc-net/net-suite/netdb-client-lib',
    install_requires=['requests', 'semver'],
    python_requires='>=3.7',
    packages=['netdb_client'],
    cmdclass={'build_py': APIGenBuild}
)

